=== modified file 'connection.cpp'
--- connection.cpp	2013-12-13 13:48:23 +0000
+++ connection.cpp	2014-02-13 13:00:30 +0000
@@ -162,6 +162,7 @@
     text.fixedProperties[TP_QT_IFACE_CHANNEL+".TargetHandleType"]  = Tp::HandleTypeContact;
     text.allowedProperties.append(TP_QT_IFACE_CHANNEL+".TargetHandle");
     text.allowedProperties.append(TP_QT_IFACE_CHANNEL+".TargetID");
+    text.allowedProperties.append(TP_QT_IFACE_CHANNEL_INTERFACE_CONFERENCE + QLatin1String(".InitialInviteeHandles"));
 
     // set requestable call channel properties
     Tp::RequestableChannelClass call;
@@ -299,6 +300,33 @@
     qDebug() << "oFonoConnection::onMMSServiceRemoved" << path;
 }
 
+oFonoTextChannel* oFonoConnection::textChannelForMembers(const QStringList &members)
+{
+    Q_FOREACH(oFonoTextChannel* channel, mTextChannels) {
+        int count = 0;
+        Tp::DBusError error;
+        if (members.size() != channel->members().size()) {
+            continue;
+        }
+        QStringList phoneNumbersOld = inspectHandles(Tp::HandleTypeContact, channel->members(), &error);
+        if (error.isValid()) {
+            continue;
+        }
+
+        Q_FOREACH(const QString &phoneNumberNew, members) {
+            Q_FOREACH(const QString &phoneNumberOld, phoneNumbersOld) {
+                if (PhoneUtils::comparePhoneNumbers(PhoneUtils::normalizePhoneNumber(phoneNumberOld), PhoneUtils::normalizePhoneNumber(phoneNumberNew))) {
+                    count++;
+                }
+            }
+        }
+        if (count == members.size()) {
+            return channel;
+        }
+    }
+    return NULL;
+}
+
 void oFonoConnection::addMMSToService(const QString &path, const QVariantMap &properties, const QString &servicePath)
 {
     qDebug() << "addMMSToService " << path << properties << servicePath;
@@ -308,12 +336,10 @@
     if (properties["Status"] ==  "received") {
         const QString normalizedNumber = PhoneUtils::normalizePhoneNumber(properties["Sender"].toString());
         // check if there is an open channel for this number and use it
-        Q_FOREACH(const QString &phoneNumber, mTextChannels.keys()) {
-            if (PhoneUtils::comparePhoneNumbers(normalizedNumber, phoneNumber)) {
-                qDebug() << "existing channel" << mTextChannels[phoneNumber];
-                mTextChannels[phoneNumber]->mmsReceived(path, properties);
-                return;
-            }
+        oFonoTextChannel *channel = textChannelForMembers(QStringList() << normalizedNumber);
+        if (channel) {
+            channel->mmsReceived(path, ensureHandle(normalizedNumber), properties);
+            return;
         }
 
         Tp::DBusError error;
@@ -325,7 +351,10 @@
             qCritical() << "Error creating channel for incoming message " << error.name() << error.message();
             return;
         }
-        mTextChannels[normalizedNumber]->mmsReceived(path, properties);
+        channel = textChannelForMembers(QStringList() << normalizedNumber);
+        if (channel) {
+            channel->mmsReceived(path, ensureHandle(normalizedNumber), properties);
+        }
     }
 }
 
@@ -533,24 +562,22 @@
 }
 
 Tp::BaseChannelPtr oFonoConnection::createTextChannel(uint targetHandleType,
-                                               uint targetHandle, Tp::DBusError *error)
+                                               uint targetHandle, const QVariantMap &hints, Tp::DBusError *error)
 {
     Q_UNUSED(targetHandleType);
-    Q_UNUSED(error);
-
-    QString newPhoneNumber = mHandles.value(targetHandle);
-
-    Q_FOREACH(const QString &phoneNumber, mTextChannels.keys()) {
-        if (PhoneUtils::comparePhoneNumbers(phoneNumber, newPhoneNumber)) {
-            return mTextChannels[phoneNumber]->baseChannel();
-        }
+
+    QStringList phoneNumbers;
+    if (hints.contains(TP_QT_IFACE_CHANNEL_INTERFACE_CONFERENCE + QLatin1String(".InitialInviteeHandles"))) {
+        phoneNumbers << inspectHandles(Tp::HandleTypeContact, qdbus_cast<Tp::UIntList>(hints[TP_QT_IFACE_CHANNEL_INTERFACE_CONFERENCE + QLatin1String(".InitialInviteeHandles")]), error);
+    } else {
+        phoneNumbers << mHandles.value(targetHandle);
     }
 
-    mTextChannels[newPhoneNumber] = new oFonoTextChannel(this, newPhoneNumber, targetHandle);
-    QObject::connect(mTextChannels[newPhoneNumber], SIGNAL(messageRead(QString)), SLOT(onMessageRead(QString)));
-    QObject::connect(mTextChannels[newPhoneNumber], SIGNAL(destroyed()), SLOT(onTextChannelClosed()));
-    qDebug() << mTextChannels[newPhoneNumber];
-    return mTextChannels[newPhoneNumber]->baseChannel();
+    oFonoTextChannel *channel = new oFonoTextChannel(this, phoneNumbers);
+    mTextChannels << channel;
+    QObject::connect(channel, SIGNAL(messageRead(QString)), SLOT(onMessageRead(QString)));
+    QObject::connect(channel, SIGNAL(destroyed()), SLOT(onTextChannelClosed()));
+    return channel->baseChannel();
 }
 
 void oFonoConnection::onMessageRead(const QString &id)
@@ -567,7 +594,7 @@
 }
 
 Tp::BaseChannelPtr oFonoConnection::createCallChannel(uint targetHandleType,
-                                               uint targetHandle, Tp::DBusError *error)
+                                               uint targetHandle, const QVariantMap &hints, Tp::DBusError *error)
 {
     Q_UNUSED(targetHandleType);
 
@@ -618,23 +645,17 @@
 }
 
 Tp::BaseChannelPtr oFonoConnection::createChannel(const QString& channelType, uint targetHandleType,
-                                               uint targetHandle, Tp::DBusError *error)
+                                               uint targetHandle, const QVariantMap &hints, Tp::DBusError *error)
 {
-    qDebug() << "oFonoConnection::createChannel" << targetHandle;
-    if( (targetHandleType != Tp::HandleTypeContact) || targetHandle == 0 || !mHandles.keys().contains(targetHandle)) {
-        error->set(TP_QT_ERROR_INVALID_HANDLE, "Handle not found");
-        return Tp::BaseChannelPtr();
-    }
-
     if (mSelfPresence.type != Tp::ConnectionPresenceTypeAvailable) {
         error->set(TP_QT_ERROR_NETWORK_ERROR, "No network available");
         return Tp::BaseChannelPtr();
     }
 
     if (channelType == TP_QT_IFACE_CHANNEL_TYPE_TEXT) {
-        return createTextChannel(targetHandleType, targetHandle, error);
+        return createTextChannel(targetHandleType, targetHandle, hints, error);
     } else if (channelType == TP_QT_IFACE_CHANNEL_TYPE_CALL) {
-        return createCallChannel(targetHandleType, targetHandle, error);
+        return createCallChannel(targetHandleType, targetHandle, hints, error);
     } else {
         error->set(TP_QT_ERROR_NOT_IMPLEMENTED, "Channel type not available");
     }
@@ -666,11 +687,11 @@
     const QString normalizedNumber = PhoneUtils::normalizePhoneNumber(pendingMessageNumber);
     PendingMessagesManager::instance()->removePendingMessage(messageId);
     // check if there is an open channel for this sender and use it
-    Q_FOREACH(const QString &phoneNumber, mTextChannels.keys()) {
-        if (PhoneUtils::comparePhoneNumbers(normalizedNumber, phoneNumber)) {
-            mTextChannels[phoneNumber]->deliveryReportReceived(messageId, info["Delivered"].toBool());
-            return;
-        }
+
+    oFonoTextChannel *channel = textChannelForMembers(QStringList() << normalizedNumber);
+    if(channel) {
+        channel->deliveryReportReceived(messageId, ensureHandle(normalizedNumber), info["Delivered"].toBool());
+        return;
     }
 
     Tp::DBusError error;
@@ -681,18 +702,21 @@
         qWarning() << "Error creating channel for incoming message" << error.name() << error.message();
         return;
     }
-    mTextChannels[normalizedNumber]->deliveryReportReceived(messageId, info["Delivered"].toBool());
+    channel = textChannelForMembers(QStringList() << normalizedNumber);
+    if(channel) {
+        channel->deliveryReportReceived(messageId, ensureHandle(normalizedNumber), info["Delivered"].toBool());
+        return;
+    }
 }
 
 void oFonoConnection::onOfonoIncomingMessage(const QString &message, const QVariantMap &info)
 {
     const QString normalizedNumber = PhoneUtils::normalizePhoneNumber(info["Sender"].toString());
     // check if there is an open channel for this sender and use it
-    Q_FOREACH(const QString &phoneNumber, mTextChannels.keys()) {
-        if (PhoneUtils::comparePhoneNumbers(normalizedNumber, phoneNumber)) {
-            mTextChannels[phoneNumber]->messageReceived(message, info);
-            return;
-        }
+    oFonoTextChannel *channel = textChannelForMembers(QStringList() << normalizedNumber);
+    if(channel) {
+        channel->messageReceived(message, ensureHandle(normalizedNumber), info);
+        return;
     }
 
     Tp::DBusError error;
@@ -703,16 +727,19 @@
         qWarning() << "Error creating channel for incoming message" << error.name() << error.message();
         return;
     }
-    mTextChannels[normalizedNumber]->messageReceived(message, info);
+
+    channel = textChannelForMembers(QStringList() << normalizedNumber);
+    if (channel) {
+        channel->messageReceived(message, ensureHandle(normalizedNumber), info);
+    }
 }
 
 void oFonoConnection::onTextChannelClosed()
 {
     oFonoTextChannel *channel = static_cast<oFonoTextChannel*>(sender());
     if (channel) {
-        QString key = mTextChannels.key(channel);
-        qDebug() << "text channel closed for number " << key;
-        mTextChannels.remove(key);
+        qDebug() << "text channel closed";
+        mTextChannels.removeAll(channel);
     }
 }
 

=== modified file 'connection.h'
--- connection.h	2013-12-13 13:48:23 +0000
+++ connection.h	2014-01-30 12:28:29 +0000
@@ -65,7 +65,7 @@
     QStringList inspectHandles(uint handleType, const Tp::UIntList& handles, Tp::DBusError *error);
     Tp::UIntList requestHandles(uint handleType, const QStringList& identifiers, Tp::DBusError* error);
     Tp::BaseChannelPtr createChannel(const QString& channelType, uint targetHandleType,
-                                     uint targetHandle, Tp::DBusError *error);
+                                     uint targetHandle,  const QVariantMap &hints, Tp::DBusError *error);
     Tp::ContactAttributesMap getContactAttributes(const Tp::UIntList &handles, const QStringList &ifaces, Tp::DBusError *error);
     uint setPresence(const QString& status, const QString& statusMessage, Tp::DBusError *error);
     void connect(Tp::DBusError *error);
@@ -87,10 +87,11 @@
     OfonoCallVolume *callVolume();
 
     uint ensureHandle(const QString &phoneNumber);
+    oFonoTextChannel* textChannelForMembers(const QStringList &members);
     Tp::BaseChannelPtr createTextChannel(uint targetHandleType,
-                                         uint targetHandle, Tp::DBusError *error);
+                                         uint targetHandle,  const QVariantMap &hints, Tp::DBusError *error);
     Tp::BaseChannelPtr createCallChannel(uint targetHandleType,
-                                         uint targetHandle, Tp::DBusError *error);
+                                         uint targetHandle,  const QVariantMap &hints, Tp::DBusError *error);
 
     ~oFonoConnection();
 
@@ -122,7 +123,7 @@
     void addMMSToService(const QString &path, const QVariantMap &properties, const QString &servicePath);
     QMap<uint, QString> mHandles;
 
-    QMap<QString, oFonoTextChannel*> mTextChannels;
+    QList<oFonoTextChannel*> mTextChannels;
     QMap<QString, oFonoCallChannel*> mCallChannels;
 
     QStringList mModems;

=== modified file 'ofonotextchannel.cpp'
--- ofonotextchannel.cpp	2013-12-13 13:48:23 +0000
+++ ofonotextchannel.cpp	2014-02-13 15:50:55 +0000
@@ -39,20 +39,27 @@
     return argument;
 }
 
-oFonoTextChannel::oFonoTextChannel(oFonoConnection *conn, QString phoneNumber, uint targetHandle, QObject *parent):
+oFonoTextChannel::oFonoTextChannel(oFonoConnection *conn, QStringList phoneNumbers, QObject *parent):
     QObject(parent),
     mConnection(conn),
-    mPhoneNumber(phoneNumber),
-    mTargetHandle(targetHandle),
+    mPhoneNumbers(phoneNumbers),
     mMessageCounter(1)
 {
     qDBusRegisterMetaType<AttachmentStruct>();
     qDBusRegisterMetaType<AttachmentList>();
 
-    Tp::BaseChannelPtr baseChannel = Tp::BaseChannel::create(mConnection,
-                                                             TP_QT_IFACE_CHANNEL_TYPE_TEXT,
-                                                             targetHandle,
-                                                             Tp::HandleTypeContact);
+    Tp::BaseChannelPtr baseChannel;
+    if (phoneNumbers.size() == 1) {
+        baseChannel = Tp::BaseChannel::create(mConnection,
+                                              TP_QT_IFACE_CHANNEL_TYPE_TEXT,
+                                              mConnection->ensureHandle(mPhoneNumbers[0]),
+                                              Tp::HandleTypeContact);
+    } else {
+        baseChannel = Tp::BaseChannel::create(mConnection,
+                                              TP_QT_IFACE_CHANNEL_TYPE_TEXT,
+                                              0,
+                                              Tp::HandleTypeNone);
+    }
     Tp::BaseChannelTextTypePtr textType = Tp::BaseChannelTextType::create(baseChannel.data());
     baseChannel->plugInterface(Tp::AbstractChannelInterfacePtr::dynamicCast(textType));
 
@@ -72,12 +79,57 @@
     mMessagesIface->setSendMessageCallback(Tp::memFun(this,&oFonoTextChannel::sendMessage));
 
     baseChannel->plugInterface(Tp::AbstractChannelInterfacePtr::dynamicCast(mMessagesIface));
+
+    mGroupIface = Tp::BaseChannelGroupInterface::create(Tp::ChannelGroupFlagCanAdd, conn->selfHandle());
+    mGroupIface->setAddMembersCallback(Tp::memFun(this,&oFonoTextChannel::onAddMembers));
+    mGroupIface->setRemoveMembersCallback(Tp::memFun(this,&oFonoTextChannel::onRemoveMembers));
+
+    baseChannel->plugInterface(Tp::AbstractChannelInterfacePtr::dynamicCast(mGroupIface));
+    addMembers(phoneNumbers);
+
     mBaseChannel = baseChannel;
     mTextChannel = Tp::BaseChannelTextTypePtr::dynamicCast(mBaseChannel->interface(TP_QT_IFACE_CHANNEL_TYPE_TEXT));
     mTextChannel->setMessageAcknowledgedCallback(Tp::memFun(this,&oFonoTextChannel::messageAcknowledged));
     QObject::connect(mBaseChannel.data(), SIGNAL(closed()), this, SLOT(deleteLater()));
 }
 
+void oFonoTextChannel::onAddMembers(const Tp::UIntList& handles, const QString& message, Tp::DBusError* error)
+{
+    addMembers(mConnection->inspectHandles(Tp::HandleTypeContact, handles, error));
+}
+
+void oFonoTextChannel::onRemoveMembers(const Tp::UIntList& handles, const QString& message, Tp::DBusError* error)
+{
+    Q_FOREACH(uint handle, handles) {
+        Q_FOREACH(const QString &phoneNumber, mConnection->inspectHandles(Tp::HandleTypeContact, Tp::UIntList() << handle, error)) {
+            mPhoneNumbers.removeAll(phoneNumber);
+        }
+        mMembers.removeAll(handle);
+    }
+    mGroupIface->removeMembers(handles);
+}
+
+void oFonoTextChannel::addMembers(QStringList phoneNumbers)
+{
+    Tp::UIntList handles;
+    Q_FOREACH(const QString &phoneNumber, phoneNumbers) {
+        uint handle = mConnection->ensureHandle(phoneNumber);
+        handles << handle;
+        if (!mPhoneNumbers.contains(phoneNumber)) {
+            mPhoneNumbers << phoneNumber;
+        }
+        if (!mMembers.contains(handle)) {
+            mMembers << handle;
+        }
+    }
+    mGroupIface->addMembers(handles, phoneNumbers);
+}
+
+Tp::UIntList oFonoTextChannel::members()
+{
+    return mMembers;
+}
+
 oFonoTextChannel::~oFonoTextChannel()
 {
 }
@@ -87,12 +139,12 @@
     return mBaseChannel;
 }
 
-void oFonoTextChannel::sendDeliveryReport(const QString &messageId, Tp::DeliveryStatus status)
+void oFonoTextChannel::sendDeliveryReport(const QString &messageId, uint handle, Tp::DeliveryStatus status)
 {
     Tp::MessagePartList partList;
     Tp::MessagePart header;
-    header["message-sender"] = QDBusVariant(mTargetHandle);
-    header["message-sender-id"] = QDBusVariant(mPhoneNumber);
+    header["message-sender"] = QDBusVariant(handle);
+    header["message-sender-id"] = QDBusVariant(mPhoneNumbers[0]);
     header["message-type"] = QDBusVariant(Tp::ChannelTextMessageTypeDeliveryReport);
     header["delivery-status"] = QDBusVariant(status);
     header["delivery-token"] = QDBusVariant(messageId);
@@ -100,9 +152,9 @@
     mTextChannel->addReceivedMessage(partList);
 }
 
-void oFonoTextChannel::deliveryReportReceived(const QString &messageId, bool success)
+void oFonoTextChannel::deliveryReportReceived(const QString &messageId, uint handle, bool success)
 {
-    sendDeliveryReport(messageId, success ? Tp::DeliveryStatusDelivered : Tp::DeliveryStatusPermanentlyFailed);
+    sendDeliveryReport(messageId, handle, success ? Tp::DeliveryStatusDelivered : Tp::DeliveryStatusPermanentlyFailed);
 }
 
 void oFonoTextChannel::messageAcknowledged(const QString &id)
@@ -116,18 +168,25 @@
     Tp::MessagePart header = message.at(0);
     Tp::MessagePart body = message.at(1);
 
-    QString objpath = mConnection->messageManager()->sendMessage(mPhoneNumber, body["content"].variant().toString(), success).path();
-    if (objpath.isEmpty() || !success) {
-        if (!success) {
-            qWarning() << mConnection->messageManager()->errorName() << mConnection->messageManager()->errorMessage();
-        } else {
-            error->set(TP_QT_ERROR_INVALID_ARGUMENT, mConnection->messageManager()->errorMessage());
+    QString objpath;
+    Q_FOREACH(const QString &phoneNumber, mPhoneNumbers) {
+        objpath = mConnection->messageManager()->sendMessage(phoneNumber, body["content"].variant().toString(), success).path();
+        // dont fail if this is a group chat as we cannot track individual messages
+        OfonoMessage *msg = new OfonoMessage(objpath);
+        QObject::connect(msg, SIGNAL(stateChanged(QString)), SLOT(onOfonoMessageStateChanged(QString)));
+        if ((objpath.isEmpty() || !success) && mPhoneNumbers.size() == 1) {
+            if (!success) {
+                qWarning() << mConnection->messageManager()->errorName() << mConnection->messageManager()->errorMessage();
+            } else {
+                error->set(TP_QT_ERROR_INVALID_ARGUMENT, mConnection->messageManager()->errorMessage());
+            }
+            msg->deleteLater();
+            continue;
         }
+        // FIXME: track pending messages only if delivery reports are enabled. We need a system config option for it.
+        PendingMessagesManager::instance()->addPendingMessage(objpath, phoneNumber);
     }
-    PendingMessagesManager::instance()->addPendingMessage(objpath, mPhoneNumber);
-
-    OfonoMessage *msg = new OfonoMessage(objpath);
-    QObject::connect(msg, SIGNAL(stateChanged(QString)), SLOT(onOfonoMessageStateChanged(QString)));
+    // return only the last one in case of group chat for history purposes
     return objpath;
 }
 
@@ -149,11 +208,11 @@
             delivery_status = Tp::DeliveryStatusUnknown;
         }
 
-        sendDeliveryReport(msg->path(), delivery_status);
+        sendDeliveryReport(msg->path(), mConnection->ensureHandle(mPhoneNumbers[0]), delivery_status);
     }
 }
 
-void oFonoTextChannel::messageReceived(const QString &message, const QVariantMap &info)
+void oFonoTextChannel::messageReceived(const QString &message, uint handle, const QVariantMap &info)
 {
     Tp::MessagePartList partList;
 
@@ -164,15 +223,15 @@
     Tp::MessagePart header;
     header["message-token"] = QDBusVariant(info["SentTime"].toString() +"-" + QString::number(mMessageCounter++));
     header["message-received"] = QDBusVariant(QDateTime::fromString(info["SentTime"].toString(), Qt::ISODate).toTime_t());
-    header["message-sender"] = QDBusVariant(mTargetHandle);
-    header["message-sender-id"] = QDBusVariant(mPhoneNumber);
+    header["message-sender"] = QDBusVariant(handle);
+    header["message-sender-id"] = QDBusVariant(mPhoneNumbers[0]);
     header["message-type"] = QDBusVariant(Tp::ChannelTextMessageTypeNormal);
     partList << header << body;
 
     mTextChannel->addReceivedMessage(partList);
 }
 
-void oFonoTextChannel::mmsReceived(const QString &id, const QVariantMap &properties)
+void oFonoTextChannel::mmsReceived(const QString &id, uint handle, const QVariantMap &properties)
 {
     Tp::MessagePartList message;
     QString subject = properties["Subject"].toString();
@@ -180,7 +239,7 @@
 
     Tp::MessagePart header;
     header["message-token"] = QDBusVariant(id);
-    header["message-sender"] = QDBusVariant(mTargetHandle);
+    header["message-sender"] = QDBusVariant(handle);
     header["message-received"] = QDBusVariant(QDateTime::fromString(properties["Date"].toString(), Qt::ISODate).toTime_t());
     header["message-type"] = QDBusVariant(Tp::DeliveryStatusDelivered);
     if (!subject.isEmpty())

=== modified file 'ofonotextchannel.h'
--- ofonotextchannel.h	2013-12-11 14:52:03 +0000
+++ ofonotextchannel.h	2014-01-23 20:47:02 +0000
@@ -34,13 +34,17 @@
 {
     Q_OBJECT
 public:
-    oFonoTextChannel(oFonoConnection *conn, QString phoneNumber, uint targetHandle, QObject *parent = 0);
+    oFonoTextChannel(oFonoConnection *conn, QStringList phoneNumbers, QObject *parent = 0);
     QString sendMessage(const Tp::MessagePartList& message, uint flags, Tp::DBusError* error);
-    void messageReceived(const QString & message, const QVariantMap &info);
+    void messageReceived(const QString & message, uint handle, const QVariantMap &info);
     Tp::BaseChannelPtr baseChannel();
     void messageAcknowledged(const QString &id);
-    void mmsReceived(const QString &id, const QVariantMap &properties);
-    void deliveryReportReceived(const QString& messageId, bool success);
+    void mmsReceived(const QString &id, uint handle, const QVariantMap &properties);
+    void deliveryReportReceived(const QString& messageId, uint handle, bool success);
+    void addMembers(QStringList phoneNumbers);
+    Tp::UIntList members();
+    void onAddMembers(const Tp::UIntList& handles, const QString& message, Tp::DBusError* error);
+    void onRemoveMembers(const Tp::UIntList& handles, const QString& message, Tp::DBusError* error);
 
 private Q_SLOTS:
     void onOfonoMessageStateChanged(QString status);
@@ -49,15 +53,16 @@
     void messageRead(const QString &id);
 
 private:
-    void sendDeliveryReport(const QString &messageId, Tp::DeliveryStatus status);
+    void sendDeliveryReport(const QString &messageId, uint handle, Tp::DeliveryStatus status);
     ~oFonoTextChannel();
     Tp::BaseChannelPtr mBaseChannel;
-    QString mPhoneNumber;
+    QStringList mPhoneNumbers;
     oFonoConnection *mConnection;
-    uint mTargetHandle;
     Tp::BaseChannelMessagesInterfacePtr mMessagesIface;
+    Tp::BaseChannelGroupInterfacePtr mGroupIface;
     Tp::BaseChannelTextTypePtr mTextChannel;
     uint mMessageCounter;
+    Tp::UIntList mMembers;
 };
 
 #endif // OFONOTEXTCHANNEL_H

=== modified file 'tests/MessagesTest.cpp'
--- tests/MessagesTest.cpp	2013-12-12 16:32:33 +0000
+++ tests/MessagesTest.cpp	2014-01-30 12:59:31 +0000
@@ -44,6 +44,7 @@
     void initTestCase();
     void testMessageReceived();
     void testMessageSend();
+    void testMessageSendGroupChat();
 
     // helper slots
     void onPendingContactsFinished(Tp::PendingOperation*);
@@ -142,6 +143,36 @@
     QCOMPARE(channel->messageQueue()[1].deliveryDetails().status(), Tp::DeliveryStatusDelivered);
 }
 
+void MessagesTest::testMessageSendGroupChat()
+{
+    // Request the contact to start chatting to
+    Tp::AccountPtr account = TelepathyHelper::instance()->account();
+    QSignalSpy spy(this, SIGNAL(contactsReceived(QList<Tp::ContactPtr>)));
+
+    connect(account->connection()->contactManager()->contactsForIdentifiers(QStringList() << "321" << "123"),
+            SIGNAL(finished(Tp::PendingOperation*)),
+            SLOT(onPendingContactsFinished(Tp::PendingOperation*)));
+
+    QTRY_COMPARE(spy.count(), 1);
+
+    QList<Tp::ContactPtr> contacts = spy.first().first().value<QList<Tp::ContactPtr> >();
+    QCOMPARE(contacts.count(), 2);
+    QCOMPARE(contacts[0]->id(), QString("321"));
+    QCOMPARE(contacts[1]->id(), QString("123"));
+
+    QSignalSpy spyTextChannel(mHandler, SIGNAL(textChannelAvailable(Tp::TextChannelPtr)));
+
+    account->createConferenceTextChat(QList<Tp::ChannelPtr>(), contacts, QDateTime::currentDateTime(), TP_QT_IFACE_CLIENT + ".TpOfonoTestHandler");
+    QTRY_COMPARE(spyTextChannel.count(), 1);
+
+    Tp::TextChannelPtr channel = spyTextChannel.first().first().value<Tp::TextChannelPtr>();
+    QVERIFY(channel);
+
+    QSignalSpy spyOfonoMessageAdded(OfonoMockController::instance(), SIGNAL(MessageAdded(QDBusObjectPath, QVariantMap)));
+    Tp::PendingSendMessage *message = channel->send("text");
+    QTRY_COMPARE(spyOfonoMessageAdded.count(), 2);
+}
+
 void MessagesTest::onPendingContactsFinished(Tp::PendingOperation *op)
 {
     Tp::PendingContacts *pc = qobject_cast<Tp::PendingContacts*>(op);

